-- PMDG 737-700 FS2020 Transponder Radio

-- Detlef von Reusner
-- V100

-- July 22 2022 - Initial Version



local gPropIllumination = user_prop_get(user_prop_add_enum("Illumination","Day,Night,Dial","Day","Select fix or left forward panel dial controlled illumination"))

local gBgDay = img_add_fullscreen("bg-day.png")

-- these light control variables are automatically set from XPlane datarefs (msfs uncapable right now)
local gDaylight = .0 
local gInteriorLight = .0
local gScalesLight = .5



local gColorDisplay = string.format("#%02x%02x%02x", 194, 194, 194)
local gColorText = string.format("#%02x%02x%02x", 237, 237, 237)

-- ================================== global variables ============================

local gTransponderMode = 0
local gTransponderFailed = false

-- =============================== Battery Energy =====================

local gEnergized = false
fs2020_variable_subscribe("ELECTRICAL AVIONICS BUS VOLTAGE", "number", function(v)
  gEnergized = (v>12)
  ShowDisplay()
end)


-- ======================== image order needed for illumination ===============

local gLightMask = img_add_fullscreen("sun-mask.png")
local gBgNight = img_add_fullscreen("bg-night.png")

local gTxtAtc = txt_add("ATC", "font:inconsolata_bold.ttf; size:15;color:"..gColorDisplay..";halign:left;", 120, 18, 65, 35)
local gTxt1 = txt_add("1", "font:inconsolata_bold.ttf; size:15;color:"..gColorDisplay..";halign:left;", 145, 18, 65, 35)
local gTxt2 = txt_add("2", "font:inconsolata_bold.ttf; size:15;color:"..gColorDisplay..";halign:left;", 152, 18, 65, 35)
local gTxtCode = txt_add("7777", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 125, 35, 65, 35)


-- ================================== Dials ATC code selector ====================

function cbDialLeftOuter(rotac)
  if rotac>0 then
    fs2020_event("ROTOR_BRAKE", 80407)
  else
    fs2020_event("ROTOR_BRAKE", 80408)
  end
end
dial_add("outer-knob.png", 84, 81, 60, 60, cbDialLeftOuter)

function cbDialLeftInner(rotac)
  if rotac>0 then
    fs2020_event("ROTOR_BRAKE", 80507)
  else
    fs2020_event("ROTOR_BRAKE", 80508)
  end
end
dial_add("outer-knob.png", 94, 91, 41, 41, cbDialLeftInner)

function cbDialRightOuter(rotac)
  if rotac>0 then
    fs2020_event("ROTOR_BRAKE", 80707)
  else
    fs2020_event("ROTOR_BRAKE", 80708)
  end
end
dial_add("outer-knob.png", 165, 81, 60, 60, cbDialRightOuter)

function cbDialRightInner(rotac)
  if rotac>0 then
    fs2020_event("ROTOR_BRAKE", 80807)
  else
    fs2020_event("ROTOR_BRAKE", 80808)
  end
end
dial_add("outer-knob.png", 175, 91, 41, 41, cbDialRightInner)



-- ===================== Ident Button =======

img_add("button-ident-pressed.png", 266, 112, 20, 20)
local gImgIdentButton = img_add("button-ident-released.png", 266, 112, 20, 20)

function cbButtonIdentPressed()
  visible(gImgIdentButton, false)
  fs2020_event("ROTOR_BRAKE", 80601)
end

function cbButtonIdentReleased()
  visible(gImgIdentButton, true)
end

button_add(nil, nil, 250, 97, 50, 50, cbButtonIdentPressed, cbButtonIdentReleased)

-- ============================= Transponder Mode  ======================

-- L:switch_800_73X Transponder mode selector, 0, 10, .., 40

local gImgDialModeSelector = img_add("selector-needle.png", 260,  31, 28, 66)
local gTestButtonPressed
local gTestButtonReleased

function ShowTransponderMode()
  img_rotate(gImgDialModeSelector, gTransponderMode / 10 * 30 - 90)
  img_rotate(gTestButtonPressed, gTransponderMode / 10 * 30 - 80)
  img_rotate(gTestButtonReleased, gTransponderMode / 10 * 30 - 80)
  ShowDisplay()
end

function cbDialModeSelector(rotac)
  if rotac>0 then
    fs2020_event("ROTOR_BRAKE", 80007)
  else
    fs2020_event("ROTOR_BRAKE", 80008)
  end
end


fs2020_variable_subscribe("L:switch_800_73X", "number", function(v) gTransponderMode=v; ShowTransponderMode() end)
dial_add(nil, 244, 35, 60, 60, cbDialModeSelector)

-- ========================== Test button ===============

gTestButtonPressed = img_add("test-button-pressed.png", 266, 57, 16, 16)
gTestButtonReleased = img_add("test-button-released.png", 266, 57, 16, 16)

function cbTestButtonPressed()
  img_visible(gTestButtonReleased, false)
  fs2020_event("ROTOR_BRAKE", 80101)
end

function cbTestButtonReleased()
  img_visible(gTestButtonReleased, true)
end

button_add(nil, nil, 259, 50, 30, 30, cbTestButtonPressed, cbTestButtonReleased)

-- ================================ Transponder selector (as button) =================

local gTransponderSelector -- 0 or 100 for 1 or 2

img_add("pointer-shadow.png", 11, 0, 70, 70)
local gImgTransponderSelector = img_add("pointer.png", 11, 0, 70, 70)

function ShowTransponderSelector()
  if gTransponderSelector == 100 then
    img_rotate(gImgTransponderSelector, 30, "LINEAR", 0.1)
  else
    img_rotate(gImgTransponderSelector, -30, "LINEAR", 0.1)
  end
  ShowDisplay()
end

button_add(nil, nil, 11, 0, 70, 70, function()
  fs2020_event("ROTOR_BRAKE", 79801)
  ShowTransponderSelector()
end)
fs2020_variable_subscribe("L:switch_798_73X", "number", function(v) gTransponderSelector = v; ShowTransponderSelector() end)

-- ================================ Alt source selector (as button) =================

local gAltSource = 0 -- 0 or 100 for 1 or 2
img_add("pointer-shadow.png", 11, 75, 70, 70)
local gImgDialAltSource = img_add("pointer.png", 11, 75, 70, 70)

function ShowAltSource()
  if gAltSource == 100 then
    img_rotate(gImgDialAltSource, 30, "LINEAR", 0.1)
  else
    img_rotate(gImgDialAltSource, -30, "LINEAR", 0.1)
  end
end

button_add(nil, nil, 11, 75, 70, 70, function()
  fs2020_event("ROTOR_BRAKE", 80301)
  ShowAltSource()
end)
fs2020_variable_subscribe("L:switch_803_73X", "number", function(v) gAltSource = v; ShowAltSource() end)

-- =========== Transponder fail light

local gImgLedFailed = img_add("led-fail-on.png", 150, 95, 10, 10)
fs2020_variable_subscribe("L:switch_799_73X", "number", function(v)
  gTransponderFailed = (v~=0)
  visible(gImgLedFailed, gEnergized and gTransponderFailed)
end)

-- ============================ Display ==============================
local gTransponderFreq = 0

function ShowDisplay()
  if gEnergized then
    visible(gTxtAtc, true)
    txt_set(gTxtCode, string.format("%04d", math.floor(gTransponderFreq)))
    if gTransponderMode > 0 then
      if gTransponderFailed then
        visible(gTxt1, false)
        visible(gTxt2, false)
      else
        if gTransponderSelector == 100 then
          visible(gTxt1, false)
          visible(gTxt2, true)
        else
          visible(gTxt1, true)
          visible(gTxt2, false)
        end
      end
    else
      visible(gTxt1, false)
      visible(gTxt2, false)
    end
  else
    txt_set(gTxtCode, "")
    visible(gTxt1, false)
    visible(gTxt2, false)
    visible(gTxtAtc, false)
  end
end

function cbFreqChange(a)
  gTransponderFreq = a
  ShowDisplay()
end

fs2020_variable_subscribe(
  "TRANSPONDER CODE:1", "number",
  cbFreqChange)

  
-- ================== illumination ========================

function cbLightChange()
  -- print("daylight: " .. gDaylight .. "   scales: ".. gScalesLight)
  local daylight = var_cap(gDaylight, .2, gDaylight)
  opacity(gLightMask, 1-daylight)
  opacity(gBgNight, gScalesLight)
end

if gPropIllumination == "Day" then
  gDaylight = .99
  gScalesLight = 0
  cbLightChange()
elseif gPropIllumination == "Night" then
  gDaylight = 0
  gScalesLight = .5
  cbLightChange()
else
  fs2020_variable_subscribe("A:LIGHT POTENTIOMETER:2", "Number",  function(v) gDaylight = v; cbLightChange() end)
  fs2020_variable_subscribe("A:LIGHT POTENTIOMETER:3", "Number",  function(v) gScalesLight = v; cbLightChange() end)
end


