-- PMDG 737-700 FS2020 Com Radio

-- Detlef von Reusner
-- V101

-- July 12 2022 - Initial Version
-- July 24 2022 - on only if avionics bus energized



local gPropIllumination = user_prop_get(user_prop_add_enum("Illumination","Day,Night,Dial","Day","Select fix or left forward panel dial controlled illumination"))

local gBgDay = img_add_fullscreen("bg-day.png")

-- these light control variables are automatically set from XPlane datarefs (msfs uncapable right now)
local gDaylight = .0 
local gInteriorLight = .0
local gScalesLight = .5



local gColorDisplay = string.format("#%02x%02x%02x", 246, 148, 49)


local gColorText = string.format("#%02x%02x%02x", 237, 237, 237)
local gButtonTextStyle = "font:arimo_regular.ttf; size:14; halign:center; color:" .. gColorText


-- ================================== global variables ============================

local gLed = {}
local gLedCount = 0
local gButton = {}
local gButtonCount = 0
local gComRadioIsOff = false

local gFreq = {}
gFreq.left = "com1_active"
gFreq.right = "com1_standby"
gFreq.com1_active = 0
gFreq.com1_standby = 0
gFreq.com2_active = 0
gFreq.com2_standby = 0

local gTimerSens = nil
local gTimerSensTics
local gTimerSensStop 

-- =============================== Battery Energy =====================

local gEnergized = false
fs2020_variable_subscribe("ELECTRICAL AVIONICS BUS VOLTAGE", "number", function(v)
  gEnergized = (v>12)
  ShowDisplays()
end)

-- ======================== image order needed for illumination ===============

local gLightMask = img_add_fullscreen("sun-mask.png")
local gBgNight = img_add_fullscreen("bg-night.png")

local gTxtActiveP1 = txt_add("987", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 42, 22, 60, 30)
local gTxtActiveP2 = txt_add("000", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 91, 22, 60, 30)

local gTxtStandbyP1 = txt_add("328", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 206, 22, 120, 30)
local gTxtStandbyP2 = txt_add("610", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 255, 22, 120, 30)

local gImgDotActive = canvas_add(85, 42, 6, 6, function()
  _circle(0, 0, 3)
  _fill(gColorDisplay)
end)
local gImgDotStandby = canvas_add(249, 42, 6, 6, function()
  _circle(0, 0, 3)
  _fill(gColorDisplay)
end)

-- ================================== Frequency knob ====================

function cbTurnOuterFreqKnob(direc)
  if gComRadioIsOff then return end
  if direc>0 then
    fs2020_event("ROTOR_BRAKE", 72607) -- com1
  else
    fs2020_event("ROTOR_BRAKE", 72608)
  end
end
dial_add("outer-knob.png", 253, 84, 60, 60, cbTurnOuterFreqKnob)


function cbTurnInnerFreqKnob(direc)
  if gComRadioIsOff then return end
  if direc>0 then
    fs2020_event("ROTOR_BRAKE", 72707) -- com1
  else
    fs2020_event("ROTOR_BRAKE", 72708)
  end
  ShowDisplays()
end
dial_add("inner-knob.png", 262, 94, 41, 41, cbTurnInnerFreqKnob)

-- ============================= HF sensitivity knob ======================

function RefreshSensDisplay()
  if gTimerSens == nil then
    gTimerSensTics = 0
    gTimerSensStop = 3 -- seconds
    gTimerSens = timer_start(0, 1000, function()
      gTimerSensTics = gTimerSensTics+1;
      if gTimerSensTics>=gTimerSensStop then
        timer_stop(gTimerSens)
        gTimerSens = nil
        ShowDisplays()
      end
    end)
  else
    gTimerSensStop = gTimerSensTics+3
  end
end
  

function cbTurnSensKnob(direc)
  if gComRadioIsOff then return end
  if direc > 0 then
    fs2020_event("ROTOR_BRAKE", 72407)
  else
    fs2020_event("ROTOR_BRAKE", 72408)
  end
  RefreshSensDisplay()
end

dial_add("left-knob.png", 31, 93, 41, 41, cbTurnSensKnob)

-- ========================= Button functions ===========================================

function ShowButton(n, pressed)
  -- x0 = 86
  -- x = x0
  -- y = 74
  -- dx = 48
  if pressed then
    -- show button n pressed
  else
    -- show button n pressed
  end
end


function ButtonPressed(n)
  local but = gButton[n]
  fs2020_event("ROTOR_BRAKE", but.event)
  -- print("Event "..but.event)
  if n==1 then -- swap frequency button
    visible(but.released, false)
    move(but.text, 66, 31)
  else
    visible(but.released, false)
    move(but.text, but.x + 2, but.y + 4)
  end
end

function ButtonReleased(n)
  local but = gButton[n]
  if n==1 then
    visible(but.released, true)
    move(but.text, 66, 30)
  else
    visible(but.released, true)
    move(but.text, but.x + 2, but.y + 3)
  end
end

function AddButton(text, event, x, y)
  gButtonCount = gButtonCount+1
  gButton[gButtonCount] = {}
  local but = gButton[gButtonCount]
  but.event = event
  if text == "SWAP" then -- transfer arrow
    but.pressed = img_add("button-pressed.png", 150, 28, 36, 18)
    but.released = img_add("button-released.png", 150, 28, 36, 18)
    but.text = img_add("swap-arrow.png", 0, 0, 130, 15)
    button_add(nil, nil, 140, 15, 60, 35, function() ButtonPressed(1) end, function() ButtonReleased(1) end)
    ButtonReleased(1)
  else
    but.x = x
    but.y = y
    but.pressed = img_add("button-pressed.png", x, y, 36, 18)
    but.released = img_add("button-released.png", x, y, 36, 18)
    -- but.txt = txt_add(text, gButtonTextStyle, x+2, y+3, 33, 30)
    but.text = txt_add(text, gButtonTextStyle, x, y, 33, 30)
    do
      local n = gButtonCount
      button_add(nil, nil, x-6, y-19, 45, 40, function() ButtonPressed(n) end, function() ButtonReleased(n) end)
      ButtonReleased(n)
    end
  end
end

-- ============================= LED functions =============================

function ComRadioIsOff()
  local k
  local is_off = true
  for k=2,gLedCount do
    if gLed[k].is_on then
      is_off = false
      break
    end
  end
  if is_off then
    txt_set(gTxtActiveP1, "")
    txt_set(gTxtActiveP2, "")
    txt_set(gTxtStandbyP1, "")
    txt_set(gTxtStandbyP2, "")
    visible(gImgDotActive, false)
    visible(gImgDotStandby, false)
    for k=2,gLedCount do visible(gLed[k].off, true) end
    gComRadioIsOff = true
    return true
  end
  gComRadioIsOff = false
  return false
end

function SwitchLed(n, on)
  -- print("set led "..n.." "..tostring(on))
  gLed[n].is_on = on
  if not ComRadioIsOff() or n==1 then
    visible(gLed[n].off, not on)
    ShowDisplays()
  end
end

function AddLed(x, y, variable, lit)
  -- function lit must return true if the led must be switched on
  gLedCount = gLedCount + 1
  gLed[gLedCount] = {}
  local led = gLed[gLedCount]
  if gLedCount == 1 then
    img_add("small-led-on.png", x, y, 10, 10)
    led.off = img_add("small-led-off.png", x, y, 10, 10)
    fs2020_variable_subscribe(variable, "number", function(v) SwitchLed(1, lit(v)) end)
  else
    img_add("led-on.png", x, y, 26, 8)
    led.off = img_add("led-off.png", x, y, 26, 8)
    do
      local n = gLedCount
      fs2020_variable_subscribe(variable, "number", function(v) SwitchLed(n, lit(v)) end)
    end
  end
end


-- ================================= Add Buttons ===================================

-- event codes from searching com behavior starting at line 31842

AddButton("SWAP", 72101, 20, 36)

x0 = 86
x = x0
y = 74
dx = 48
dy = 42

AddButton("VHF1", 90401, x, y)

x = x + dx 
AddButton("VHF2", 90601, x, y)

x = x + dx 
AddButton("VHF3", 90801, x, y)

x = x0
y = y + dy
AddButton("HF1", 91001, x, y)

x = x + dx 
AddButton("AM", 91201, x, y)

x = x + dx 
AddButton("HF2", 91401, x, y)

-- not used: 723 - VHF Outer Frequency Selector Knob ##
AddButton("OFF", 90301, 31, 63) -- ok -- 72401 == HF Sense knob push
AddButton("TEST", 72501, 237, 56)

-- =================================== Add LEDs ===============================

AddLed(164, 10, "L:switch_720_73X", function(v) return v>0 end) -- offside tuning light, n==1

x0 = 92
x = x0
y = 59

AddLed(x, y, "L:switch_9041_73X", function(v) return v>0 end) -- vhf1
x = x + dx

AddLed(x, y, "L:switch_9042_73X", function(v) return v>0 end) -- vhf2
x = x + dx

AddLed(x, y, "L:switch_9043_73X", function(v) return v>0 end) -- vhf3
x = x + dx

x = x0
y = y + dy
AddLed(x, y, "L:switch_9044_73X", function(v) return v>0 end) -- hf1
x = x + dx

AddLed(x, y, "L:switch_9045_73X", function(v) return v>0 end) -- am
x = x + dx

AddLed(x, y, "L:switch_9046_73X", function(v) return v>0 end) -- hf2

-- ============================ Displays ==============================


function MakeSensString(v)
  return string.format("%03d", math.floor(v))
end

function MakeFrequencyString(v)
  local p1, p2
  v = (v+1)/1000000
  p1 = math.floor(v)
  p2 = math.floor((v-p1)*1000)
  return string.format("%03d", p1), string.format("%03d", p2)
end

function ShowDisplays()
  -- print("led 2 is "..tostring(gLed[2].is_on))
  -- print("led 3 is "..tostring(gLed[3].is_on))

  if not gEnergized then
    txt_set(gTxtActiveP1, "")
    txt_set(gTxtActiveP2, "")
    txt_set(gTxtStandbyP1, "")
    txt_set(gTxtStandbyP2, "")
    visible(gImgDotActive, false)
    visible(gImgDotStandby, false)
    return
  end


  local p1, p2

  if gTimerSens ~= nil then
    gFreq.right = "sens"
    p1 = MakeSensString(gFreq.sens)
    txt_set(gTxtStandbyP1, p1)
    txt_set(gTxtStandbyP2, "")
    visible(gImgDotActive, true)
    visible(gImgDotStandby, false)
    if gLed[2].is_on then
      gFreq.left = "com1_active"
    elseif gLed[3].is_on then
      gFreq.left = "com2_active"
    else
      txt_set(gTxtActiveP1, "")
      txt_set(gTxtActiveP2, "")
      txt_set(gTxtStandbyP1, "")
      txt_set(gTxtStandbyP2, "")
      return
    end
    p1, p2 = MakeFrequencyString(gFreq[gFreq.left])
    txt_set(gTxtActiveP1, p1)
    txt_set(gTxtActiveP2, p2)
    
  else
    visible(gImgDotActive, true)
    visible(gImgDotStandby, true)
    if gLed[2].is_on then
      gFreq.left = "com1_active"
      gFreq.right = "com1_standby"
    elseif gLed[3].is_on then
      gFreq.left = "com2_active"
      gFreq.right = "com2_standby"
    else
      txt_set(gTxtActiveP1, "")
      txt_set(gTxtActiveP2, "")
      txt_set(gTxtStandbyP1, "")
      txt_set(gTxtStandbyP2, "")
      return
    end
    p1, p2 = MakeFrequencyString(gFreq[gFreq.left])
    txt_set(gTxtActiveP1, p1)
    txt_set(gTxtActiveP2, p2)

    p1, p2 = MakeFrequencyString(gFreq[gFreq.right])
    txt_set(gTxtStandbyP1, p1)
    txt_set(gTxtStandbyP2, p2)
  end
end

function cbFreqChange(a, b, c, d, e, f, g)
  gFreq.com1_standby = a
  gFreq.com1_active = b
  gFreq.com2_standby = c
  gFreq.com2_active = d
  gFreq.com3_standby = e
  gFreq.com3_active = f
  gFreq.sens = g
  ShowDisplays()
end

fs2020_variable_subscribe(
  "COM STANDBY FREQUENCY:1", "number", 
  "COM ACTIVE FREQUENCY:1", "number",
  "COM STANDBY FREQUENCY:2", "number", 
  "COM ACTIVE FREQUENCY:2", "number",
  "COM STANDBY FREQUENCY:3", "number", 
  "COM ACTIVE FREQUENCY:3", "number",
  "L:switch_724_73X", "number",
  cbFreqChange)

-- ================== illumination ========================

function cbLightChange()
  -- print("daylight: " .. gDaylight .. "   scales: ".. gScalesLight)
  local daylight = var_cap(gDaylight, .2, gDaylight)
  opacity(gLightMask, 1-daylight)
  opacity(gBgNight, gScalesLight)
end

if gPropIllumination == "Day" then
  gDaylight = .99
  gScalesLight = 0
  cbLightChange()
elseif gPropIllumination == "Night" then
  gDaylight = 0
  gScalesLight = .5
  cbLightChange()
else
  fs2020_variable_subscribe("A:LIGHT POTENTIOMETER:2", "Number",  function(v) gDaylight = v; cbLightChange() end)
  fs2020_variable_subscribe("A:LIGHT POTENTIOMETER:3", "Number",  function(v) gScalesLight = v; cbLightChange() end)
  
end


