-- PMDG 737-700 FS2020 Nav Radio

-- Detlef von Reusner
-- V101

-- July 14 2022 - Initial Version
-- July 20 2022 - Nav2 user prop added
-- July 24 2022 - on only if avionics bus energized




local gPropIllumination = user_prop_get(user_prop_add_enum("Illumination","Day,Night,Dial","Day","Select fix or left forward panel dial controlled illumination"))

local gPropNavSelect = user_prop_get(user_prop_add_enum("NAV radio","NAV1,NAV2", "NAV1", "Select radio"))

gSelect = {}

if gPropNavSelect == "NAV1" then
  gSelect.event_inc_whole = 73307
  gSelect.event_dec_whole = 73308
  gSelect.event_inc_fract = 73207
  gSelect.event_dec_fract = 73208
  gSelect.event_swap = 72901
  gSelect.lvar_active = "NAV ACTIVE FREQUENCY:1"
  gSelect.lvar_standby = "NAV STANDBY FREQUENCY:1"
else
  gSelect.event_inc_whole = 84807
  gSelect.event_dec_whole = 84808
  gSelect.event_inc_fract = 84907
  gSelect.event_dec_fract = 84908
  gSelect.event_swap = 84501
  gSelect.lvar_active = "NAV ACTIVE FREQUENCY:2"
  gSelect.lvar_standby = "NAV STANDBY FREQUENCY:2"
end


local gBgDay = img_add_fullscreen("bg-day.png")

-- these light control variables are automatically set from XPlane datarefs (msfs uncapable right now)
local gDaylight = .0 
local gInteriorLight = .0
local gScalesLight = .5



local gColorDisplay = string.format("#%02x%02x%02x", 246, 148, 49)


local gColorText = string.format("#%02x%02x%02x", 237, 237, 237)


-- ================================== global variables ============================

local gFreq = {}
gFreq.active = 0
gFreq.standby = 0

local gImgButtonReleased
local gButtonText

-- =============================== Battery Energy =====================

local gEnergized = false
fs2020_variable_subscribe("ELECTRICAL AVIONICS BUS VOLTAGE", "number", function(v)
  gEnergized = (v>12)
  ShowDisplays()
end)

-- ======================== image order needed for illumination ===============

local gLightMask = img_add_fullscreen("sun-mask.png")
local gBgNight = img_add_fullscreen("bg-night.png")

local gTxtActiveP1 = txt_add("987", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 42, 22, 60, 30)
local gTxtActiveP2 = txt_add("000", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 91, 22, 60, 30)

local gTxtStandbyP1 = txt_add("328", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 206, 22, 120, 30)
local gTxtStandbyP2 = txt_add("610", "font:digital-7-mono.ttf; size:27;color:"..gColorDisplay..";halign:left;", 255, 22, 120, 30)

local gImgDotActive = canvas_add(85, 42, 6, 6, function()
  _circle(0, 0, 3)
  _fill(gColorDisplay)
end)
local gImgDotStandby = canvas_add(249, 42, 6, 6, function()
  _circle(0, 0, 3)
  _fill(gColorDisplay)
end)

-- ========================= Button functions ===========================================

function ButtonPressed()
  fs2020_event("ROTOR_BRAKE", gSelect.event_swap)
  visible(gImgButtonReleased, false)
  move(gButtonText, 66, 31)
end

function ButtonReleased()
  visible(gImgButtonReleased, true)
  move(gButtonText, 66, 30)
end

img_add("button-pressed.png", 150, 28, 36, 18)
gImgButtonReleased = img_add("button-released.png", 150, 28, 36, 18)
gButtonText = img_add("swap-arrow.png", 0, 0, 130, 15)
button_add(nil, nil, 140, 15, 60, 35, ButtonPressed, ButtonReleased)
ButtonReleased()

-- ================================== Frequency knob ====================

function cbTurnOuterFreqKnob(direc)
  if direc>0 then
    fs2020_event("ROTOR_BRAKE", gSelect.event_inc_whole)
  else
    fs2020_event("ROTOR_BRAKE", gSelect.event_dec_whole)
  end
end
dial_add("outer-knob.png", 253, 84, 60, 60, cbTurnOuterFreqKnob)


function cbTurnInnerFreqKnob(direc)
  if direc>0 then
    fs2020_event("ROTOR_BRAKE", gSelect.event_inc_fract)
  else
    fs2020_event("ROTOR_BRAKE", gSelect.event_dec_fract)
  end
  ShowDisplays()
end
dial_add("inner-knob.png", 262, 94, 41, 41, cbTurnInnerFreqKnob)

button_add(nil, nil, 282, 114, 20, 20, ButtonPressed, ButtonReleased)

-- ============================ Displays ==============================

function MakeFrequencyString(v)
  local p1, p2
  v = (v+1)/1000000
  p1 = math.floor(v)
  p2 = math.floor((v-p1)*1000)
  return string.format("%03d", p1), string.format("%03d", p2)
end

function ShowDisplays()
  local p1, p2

  if not gEnergized then
    visible(gImgDotActive, false)
    visible(gImgDotStandby, false)
    txt_set(gTxtActiveP1, "")
    txt_set(gTxtActiveP2, "")

    txt_set(gTxtStandbyP1, "")
    txt_set(gTxtStandbyP2, "")
    return
  end

  visible(gImgDotActive, true)
  visible(gImgDotStandby, true)
  p1, p2 = MakeFrequencyString(gFreq.active)
  txt_set(gTxtActiveP1, p1)
  txt_set(gTxtActiveP2, p2)

  p1, p2 = MakeFrequencyString(gFreq.standby)
  txt_set(gTxtStandbyP1, p1)
  txt_set(gTxtStandbyP2, p2)
end

function cbFreqChange(a, b)
  gFreq.standby = a
  gFreq.active = b
  ShowDisplays()
end

fs2020_variable_subscribe(
  gSelect.lvar_standby, "number", 
  gSelect.lvar_active, "number",
  cbFreqChange)

-- ================== illumination ========================

function cbLightChange()
  -- print("daylight: " .. gDaylight .. "   scales: ".. gScalesLight)
  local daylight = var_cap(gDaylight, .2, gDaylight)
  opacity(gLightMask, 1-daylight)
  opacity(gBgNight, gScalesLight)
end

if gPropIllumination == "Day" then
  gDaylight = .99
  gScalesLight = 0
  cbLightChange()
elseif gPropIllumination == "Night" then
  gDaylight = 0.2
  gScalesLight = .5
  cbLightChange()
else
  fs2020_variable_subscribe("A:LIGHT POTENTIOMETER:2", "Number",  function(v) gDaylight = v; cbLightChange() end)
  fs2020_variable_subscribe("A:LIGHT POTENTIOMETER:3", "Number",  function(v) gScalesLight = v; cbLightChange() end)
  
end
